//
//  AppDelegate.m
//  GamesWithTeams
//
//  Created by Traun Leyden on 3/16/13.
//  Copyright (c) 2013 TL. All rights reserved.
//

#import "AppDelegate.h"

#import "GameListController.h"
#import <CouchbaseLite/CouchbaseLite.h>
#import "GameStore.h"
#import "SyncManager.h"
#import "BrowserIDController+UIKit.h"

AppDelegate* gAppDelegate;
NSString *serverUrl = @"http://127.0.0.1:5984/games-with-teams";  // CouchDB or Sync Gateway URL

@interface AppDelegate () <SyncManagerDelegate, BrowserIDControllerDelegate>
@end

@implementation AppDelegate
{
    CBLDatabase* _database;
    GameStore* _gameStore;
    SyncManager* _syncManager;
    BrowserIDController* _browserIDController;
    UINavigationController* _navigationController;
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    gAppDelegate = self;

    [self initDatabase];
    
    [self setupSync];
    
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    // Override point for customization after application launch.
    GameListController *controller = [[GameListController alloc] initWithNibName:@"GameListController" bundle:nil];
    _navigationController = [[UINavigationController alloc] initWithRootViewController:controller];
    self.window.rootViewController = _navigationController;
    [self.window makeKeyAndVisible];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

- (void)initDatabase {
    
    // Initialize CouchbaseLite:
    NSError* error;
    _database = [[CBLManager sharedInstance] createDatabaseNamed: @"games_with_teams"
                                                           error: &error];
    if (!_database)
        [self showAlert: @"Couldn't open database" error: error fatal: YES];
    
    _gameStore = [[GameStore alloc] initWithDatabase:_database];
    
}


#pragma mark - SYNC & LOGIN:


- (void) setupSync {
    _syncManager = [[SyncManager alloc] initWithDatabase: _database];
    _syncManager.delegate = self;
    _syncManager.continuous = YES;
    _syncManager.syncURL = [NSURL URLWithString: serverUrl];
}


- (void) syncManagerProgressChanged: (SyncManager*)manager {
    if (_gameStore.username == nil) {
        CBLReplication* repl = manager.replications[0];
        if (repl.mode == kCBLReplicationIdle) {
            // Pick up my username from the replication, on the first sync:
            NSString* username = repl.browserIDEmailAddress;
            if (!username)
                username = repl.credential.user;
            if (username) {
                NSLog(@"Game username = '%@'", username);
                _gameStore.username = username;
            }
        }
    }
}


- (bool) syncManagerShouldPromptForLogin: (SyncManager*)manager {
    [self showAlert:@"SyncManager connection error" error:nil fatal:YES];
    return NO;
}


- (void) browserIDControllerDidCancel: (BrowserIDController*) browserIDController {
    [_browserIDController.viewController dismissViewControllerAnimated: YES completion: NULL];
    _browserIDController = nil;
}

- (void) browserIDController: (BrowserIDController*) browserIDController
           didFailWithReason: (NSString*) reason
{
    [self browserIDControllerDidCancel: browserIDController];
}

- (void) browserIDController: (BrowserIDController*) browserIDController
     didSucceedWithAssertion: (NSString*) assertion
{
    [self browserIDControllerDidCancel: browserIDController];
    for (CBLReplication* repl in _syncManager.replications) {
        [repl registerBrowserIDAssertion: assertion];
    }
}


#pragma mark - ALERT:


// Display an error alert, without blocking.
// If 'fatal' is true, the app will quit when it's pressed.
- (void)showAlert: (NSString*)message error: (NSError*)error fatal: (BOOL)fatal {
    if (error) {
        message = [NSString stringWithFormat: @"%@\n\n%@", message, error.localizedDescription];
    }
    UIAlertView* alert = [[UIAlertView alloc] initWithTitle: (fatal ? @"Fatal Error" : @"Error")
                                                    message: message
                                                   delegate: (fatal ? self : nil)
                                          cancelButtonTitle: (fatal ? @"Quit" : @"Sorry")
                                          otherButtonTitles: nil];
    [alert show];
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
    exit(0);
}

@end
