//
//  GameListController.m
//  GamesWithTeams
//
//  Created by Traun Leyden on 3/16/13.
//  Copyright (c) 2013 TL. All rights reserved.
//

#import "GameListController.h"
#import "GameWithTeams.h"
#import "GameController.h"
#import "GameStore.h"
#import "AppDelegate.h"
#import "UIAlertView+Blocks.h"

@interface GameListController ()

@end

@implementation GameListController {
    IBOutlet UITableView* _table;
    NSArray* _games;

}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [[GameStore sharedInstance] addObserver: self forKeyPath: @"allGames"
                    options: NSKeyValueObservingOptionInitial context: NULL];
    
    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
                                                                            target:self
                                                                            action:@selector(addGameButtonPressed:)];
    
    self.navigationItem.rightBarButtonItem = addButton;
    
    if ( [self isUserLoggedIn] == NO )
        [self promptUserLogin];
    else
        [self customizeTitleWithUsername];
        
    
}

- (void)dealloc
{
    [[GameStore sharedInstance] removeObserver: self forKeyPath: @"allGames"];
}

- (void) customizeTitleWithUsername {
    
    self.navigationItem.title = [[GameStore sharedInstance] username];
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - BUTTON ACTIONS:


- (void) addGameButtonPressed: (id)sender
{
    NSString *title = [NSString stringWithFormat: @"game %@", [NSDate date]];
    GameWithTeams *gameWithTeams = [[GameStore sharedInstance] newGameWithTitle: title];
    
    NSError* error;
    if (![gameWithTeams save: &error]) {
        [gAppDelegate showAlert: @"Couldn't create game" error: error fatal: NO];
    }
    
}


#pragma mark - USER:

- (void) promptUserLogin {
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"No username found"
                                                    message:@"Enter your username..."
                                                   delegate:self
                                          cancelButtonTitle:@"Cancel"
                                          otherButtonTitles:@"Ok", nil];
    
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    
    [alert show];
    
}


- (BOOL) isUserLoggedIn {
    
    return [[GameStore sharedInstance] username] != nil;
}

- (void) alertView: (UIAlertView *)alertView didDismissWithButtonIndex: (NSInteger)buttonIndex

{
    if (buttonIndex == 1) {
        NSString *username = [[alertView textFieldAtIndex:0] text];
        [[GameStore sharedInstance] setUsername:username];
        [self customizeTitleWithUsername];
    }
    
}

- (void) promptUserToChooseTeamForGame: (GameWithTeams*)gameWithTeams withBlock:(void(^)())block
{
    UIAlertView *alertView;
    RIButtonItem *whiteTeamButtonItem;
    RIButtonItem *blackTeamButtonItem;
    
    whiteTeamButtonItem = [RIButtonItem item];
    blackTeamButtonItem  = [RIButtonItem item];
    
    [whiteTeamButtonItem setLabel:@"White"];
    [whiteTeamButtonItem setAction:^ {
        NSLog(@"chose white");
        
        NSString *username = [[GameStore sharedInstance] username];
        [gameWithTeams addUser: username toTeam: WhiteTeam];
        NSError* error;
        if (![gameWithTeams save: &error]) {
            [gAppDelegate showAlert: @"Couldn't save user to team" error: error fatal: NO];
        }
        block();
    }];
	
    [blackTeamButtonItem setLabel:@"Black"];
    [blackTeamButtonItem setAction:^ {
        NSString *username = [[GameStore sharedInstance] username];
        [gameWithTeams addUser: username toTeam: BlackTeam];
        NSError* error;
        if (![gameWithTeams save: &error]) {
            [gAppDelegate showAlert: @"Couldn't save user to team" error: error fatal: NO];
        }
        block();
    }];
    
    alertView = [[UIAlertView alloc] initWithTitle:@"Choose Team"
                                           message:@"Pick which team you want to play on."
                                  cancelButtonItem:nil
                                  otherButtonItems:whiteTeamButtonItem, blackTeamButtonItem, nil];
    
    [alertView show];
    
    
}


#pragma mark - KVO:

- (void) observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object
                         change:(NSDictionary *)change context:(void *)context
{
    if (object == [GameStore sharedInstance]) {
        _games = [[GameStore sharedInstance] allGames];
        [_table reloadData];
    } 
}


#pragma mark - TABLE DISPLAY:


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _games.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier: @"Game"];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle: UITableViewCellStyleSubtitle
                                      reuseIdentifier: @"Game"];
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    GameWithTeams *game = [_games objectAtIndex:indexPath.row];
    [self updateCell: cell forGame: game];
    return cell;
}

- (void) updateCell: (UITableViewCell*)cell forGame: (GameWithTeams*)game {
    
    cell.textLabel.text = game.title;
}


#pragma mark - SELECTION:

- (void) showGameControllerForGame: (GameWithTeams*)gameWithTeams
{
    GameController *controller;
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        controller = [[GameController alloc] initWithNibName:@"GameController_iPhone" bundle:nil game:gameWithTeams];
    } else {
        controller = [[GameController alloc] initWithNibName:@"GameController_iPad" bundle:nil game:gameWithTeams];
    }
    
    [[self navigationController] pushViewController:controller animated:YES];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    GameWithTeams *gameWithTeams = [_games objectAtIndex:indexPath.row];

    NSString *username = [[GameStore sharedInstance] username];
    TeamId teamId = [gameWithTeams teamForUser: username];
    
    if ( teamId == NoTeam ) {
        [self promptUserToChooseTeamForGame: gameWithTeams withBlock:^{
            [self showGameControllerForGame:gameWithTeams];
        }];
    }
    else {
        [self showGameControllerForGame:gameWithTeams];

    }

    
}


@end
