//
//  GameStore.m
//  GamesWithTeams
//
//  Created by Traun Leyden on 3/16/13.
//  Copyright (c) 2013 TL. All rights reserved.
//

#import "GameStore.h"
#import "GameWithTeams.h"

static GameStore* sInstance;

@interface GameStore ()
@property (readwrite, copy) NSArray* allGames;
@end


@implementation GameStore {
    CBLLiveQuery* _gamesQuery;
}

@synthesize username=_username, allGames=_allGames;

+ (GameStore*) sharedInstance {
    return sInstance;
}

- (id) initWithDatabase: (CBLDatabase*)database {
    self = [super init];
    if (self) {
        NSAssert(!sInstance, @"Cannot create more than one GameStore");
        sInstance = self;
        _database = database;
        _username = [[NSUserDefaults standardUserDefaults] stringForKey: @"UserName"];

        [_database.modelFactory registerClass: [GameWithTeams class] forDocumentType: @"game_with_teams"];
        
        [self initViews];
    }
    return self;
}

- (void) initViews {
    CBLView* view = [_database viewNamed: @"games"];
    [view setMapBlock: MAPBLOCK({
        if ([doc[@"type"] isEqualToString: @"game_with_teams"]) {
            emit(doc[@"_id"], doc);
        }
    }) version: @"1"];
    
    _gamesQuery = [[view query] asLiveQuery];
    [_gamesQuery addObserver: self forKeyPath: @"rows"
                            options: NSKeyValueObservingOptionInitial context: NULL];

    
}

- (void) setUsername:(NSString *)username {
    if (![username isEqualToString: _username]) {
        NSLog(@"Setting username to '%@'", username);
        _username = [username copy];
        [[NSUserDefaults standardUserDefaults] setObject: username forKey: @"UserName"];
        [[NSUserDefaults standardUserDefaults] synchronize];

    }
}


- (void)dealloc
{
    [_gamesQuery removeObserver: self forKeyPath: @"rows"];
}


- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if (object == _gamesQuery) {
        [self refreshGameList];
    } else {
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}


#pragma mark - GAMES:

- (GameWithTeams*) newGameWithTitle: (NSString*)title {
    return [[GameWithTeams alloc] initNewWithTitle: title inGameStore: self whitePlayer:self.username];
}

- (void) refreshGameList {
    NSMutableArray* games = [NSMutableArray array];
    for (CBLQueryRow* row in _gamesQuery.rows) {
        CBLDocument* document = [_database documentWithID: row.key0];
        GameWithTeams* game = [GameWithTeams modelForDocument: document];
        [games addObject: game];
    }
    if (![games isEqual: _allGames]) {
        self.allGames = games;
    }
}



@end
