//
//  GameWithTeams.h
//  GamesWithTeams
//
//  Created by Traun Leyden on 3/16/13.
//  Copyright (c) 2013 TL. All rights reserved.
//

#import <CouchbaseLite/CouchbaseLite.h>

@class UserProfile, GameStore;

typedef enum {
    WhiteTeam = 0,
    BlackTeam,
    NoTeam
} TeamId;

@interface GameWithTeams : CBLModel

/** Create a new game */
- (id) initNewWithTitle: (NSString*)title
            inGameStore: (GameStore*)gameStore
            whitePlayer: (NSString*)username;


/** Record move by player and update who should make the next move */
- (void) addMove: (NSString*)move
          byTeam: (TeamId)teamId;

/** Update the last player to move for a given team */
- (void) updateLastUserToMove: (NSString*)username onTeam: (TeamId)teamId;

/** Add user to team */
- (void) addUser: (NSString*)username toTeam: (TeamId)teamId;

/** The username who's turn it is to move */
- (NSString*) nextUserToMove;

/** workaround for the fact that CBLite is treating an empty array as null/NSNull */
- (NSArray *) turnsNonNull;

/** Which team does this user belong to? */
- (TeamId) teamForUser: (NSString*)username;

/** The title of the game, eg, "my game" */
@property (readwrite) NSString* title;

/** eg, ["p1_id", "p2_id"] */
@property (copy) NSArray* white_team;

/** eg, ["p3_id", "p4_id"] */
@property (copy) NSArray* black_team;

/** eg, "ChessGame" */
@property (readonly) NSString* game_type;

/** array of turns */
@property (copy) NSArray* turns;

/** eg, 0 for WhiteTeam, 1 for BlackTeam */
@property (readwrite) NSNumber* next_turn_team_id;

/** The player who made the last move for the white team, eg, "p1" */
@property (readwrite) NSString* last_white_move_player;

/** The player who made the last move for the black team, eg, "p2" */
@property (readwrite) NSString* last_black_move_player;

/** eg, "started"  (| "created" | "finished") */
@property (readwrite) NSString* game_state;

@end
