//
//  GameWithTeams.m
//  GamesWithTeams
//
//  Created by Traun Leyden on 3/16/13.
//  Copyright (c) 2013 TL. All rights reserved.
//

#import "GameWithTeams.h"
#import "GameStore.h"


@implementation GameWithTeams

@dynamic title, white_team, black_team, game_type,
next_turn_team_id, game_state, turns, last_white_move_player, last_black_move_player;

// New-document initializer
- (id) initNewWithTitle: (NSString*)title
            inGameStore: (GameStore*)gameStore
            whitePlayer: (NSString*)username
{
    self = [super initWithNewDocumentInDatabase: gameStore.database];
    if (self) {
        [self setValue: @"game_with_teams" ofProperty: @"type"];
        self.title = title;
        [self setTurns:[NSArray array]];
        [self setWhite_team:[NSArray arrayWithObject:username]];
        [self setNext_turn_team_id:[NSNumber numberWithInt:WhiteTeam]];
    }
    return self;
}

#pragma mark - PLAYERS:

- (TeamId) teamForUser: (NSString*)username
{
    if ( [self.white_team containsObject:username] )
        return WhiteTeam;
    else if ( [self.black_team containsObject:username] )
        return BlackTeam;
    else
        return NoTeam;
}

- (void) addUser: (NSString*)username toTeam: (TeamId)teamId
{
    if ( teamId == WhiteTeam ) {
        NSMutableArray *whiteTeamMembers = [[self whiteTeamMembersNonNull] mutableCopy];
        [whiteTeamMembers addObject:username];
        [self setWhite_team:whiteTeamMembers];
    }
    else {
        NSMutableArray *blackTeamMembers = [[self blackTeamMembersNonNull] mutableCopy];
        [blackTeamMembers addObject:username];
        [self setBlack_team:blackTeamMembers];
        
    }
}

- (NSString*) nextUserToMove {
    
    TeamId teamIdNextMove = [[self next_turn_team_id] intValue];
    NSArray *teamNextMove;
    NSString *lastPlayerToMove;
    if ( teamIdNextMove == WhiteTeam ) {
        teamNextMove = [self whiteTeamMembersNonNull];
        lastPlayerToMove = [self last_white_move_player];
    }
    else {
        teamNextMove = [self blackTeamMembersNonNull];
        lastPlayerToMove = [self last_black_move_player];
    }
    

    
    // find the index of the next player in the array, or wrap around to start if we are at the end
    if ( [teamNextMove count] > 0 ) {

        int indexNextPlayer;
        if ( lastPlayerToMove == nil ) {
            indexNextPlayer = 0;
        }
        else {
            int indexLastPlayer = [teamNextMove indexOfObject:lastPlayerToMove];
            if ( indexLastPlayer == [teamNextMove count] -1 )
                indexNextPlayer = 0;
            else
                indexNextPlayer = indexLastPlayer + 1;
        }

        return [teamNextMove objectAtIndex:indexNextPlayer];

        
    }
    else {
        return @"Error";
    }
    
}


#pragma mark - MOVES:
- (void) addMove: (NSString*)move byTeam: (TeamId)teamId
{

    [self updateNextTeamToMove: teamId];
    
    NSDictionary *turnDictionary = @{@"move" : move, @"team_id": [NSNumber numberWithInt:teamId]};
    
    // add the turn to the array of turns
    NSMutableArray *turns = [[self turnsNonNull] mutableCopy];
    [turns addObject:turnDictionary];
    [self setTurns:turns];

}

- (void) updateLastUserToMove: (NSString*)username onTeam: (TeamId)teamId
{
    if ( teamId == WhiteTeam )
        [self setLast_white_move_player:username];
    else
        [self setLast_black_move_player:username];
}

- (NSArray *) turnsNonNull
{
    BOOL isNotNull = [[self turns] isKindOfClass:[NSNull class]] == NO;
    if ( isNotNull )
        return [self turns];
    else
        return [NSArray array];
    
}

- (NSArray *) whiteTeamMembersNonNull
{
    BOOL isNotNull = [[self white_team] isKindOfClass:[NSNull class]] == NO;
    if ( isNotNull && [self white_team] != nil )
        return [self white_team];
    else
        return [NSArray array];
    
}

- (NSArray *) blackTeamMembersNonNull
{
    BOOL isNotNull = [[self black_team] isKindOfClass:[NSNull class]] == NO;
    if ( isNotNull && [self black_team] != nil )
        return [self black_team];
    else
        return [NSArray array];
    
}


- (void) updateNextTeamToMove: (TeamId)currentMoveTeamId
{
    // find out the opposite team of this player
    if ( currentMoveTeamId == WhiteTeam )
        [self setNext_turn_team_id: [NSNumber numberWithInt: BlackTeam]];
    else
        [self setNext_turn_team_id: [NSNumber numberWithInt: WhiteTeam]];
    
}


@end
